@@ -21,8 +21,8 @@
 import logging
 from configparser import ConfigParser
 from datetime import datetime
-from os import listdir, makedirs, walk
-from os.path import exists, join
+from os import access, chmod, listdir, makedirs, walk, X_OK
+from os.path import abspath, exists, isdir, join
 from shutil import copy
 
 from pypi2deb import VERSION, OVERRIDES_PATH, PROFILES_PATH, TEMPLATES_PATH
@@ -31,9 +31,9 @@ from pypi2deb.tools import execute
 from jinja2 import Environment, FileSystemLoader
 from debian.changelog import Changelog, Version, get_maintainer
 try:
-    from simplejson import load
+    from simplejson import load, dump
 except ImportError:
-    from json import load
+    from json import load, dump
 
 from dhpython import PKG_PREFIX_MAP
 from dhpython.pydist import guess_dependency, parse_pydep
@@ -43,6 +43,8 @@ log = logging.getLogger('pypi2deb')
 SPHINX_DIR = {'docs', 'doc', 'doc/build'}
 INTERPRETER_MAP = {value: key for key, value in PKG_PREFIX_MAP.items()}
 VERSIONED_I_MAP = {'python': 'python2'}
+DESC_STOP_KEYWORDS = {'changelog', 'changes', 'license', 'requirements',
+                      'installation'}
 
 
 def _copy_static_files(src_dir, debian_dir):
@@ -68,41 +70,66 @@ def debianize(dpath, ctx, profile=None):
         if 'py2dsp' in upstream_cfg:
             ctx.update(upstream_cfg['py2dsp'].items())
 
+    override_paths = [TEMPLATES_PATH]
+
+    # profile overrides
     if profile:
-        if exists(profile):
+        if isdir(profile):  # --profile a_dir
+            override_paths.append(profile)
+        elif exists(profile):  # --profile a_file
             with open(profile) as fp:
                 ctx.update(load(fp))
-        else:
-            profile_fpath = join(PROFILES_PATH, profile, 'ctx.json')
-            if exists(profile_fpath):
-                with open(profile_fpath) as fp:
-                    ctx.update(load(fp))
+        else:  # --profile name
+            profile_dpath = join(PROFILES_PATH, profile)
+            if isdir(profile_dpath):
+                override_paths.append(profile_dpath)
 
-    override_fpath = join(OVERRIDES_PATH, ctx['name'].lower(), 'ctx.json')
-    if exists(override_fpath):
-        with open(override_fpath) as fp:
-            ctx.update(load(fp))
+    # package specific overrides... in global overrides dir
+    o_dpath = join(OVERRIDES_PATH, ctx['name'].lower())
+    isdir(o_dpath) and override_paths.append(o_dpath)
+    # ... in local ./overrides
+    o_dpath = join('overrides', ctx['name'].lower())
+    isdir(o_dpath) and override_paths.append(o_dpath)
+
+    # handle overrides
+    debian_dir = join(dpath, 'debian')
+    for o_dpath in override_paths:
+        fpath = join(o_dpath, 'ctx.json')
+        if exists(fpath):
+            with open(fpath) as fp:
+                ctx.update(load(fp))
+        # invoke pre hooks
+        fpath = abspath(join(o_dpath, 'hooks', 'pre'))
+        if exists(fpath) and access(fpath, X_OK):
+            _dump_ctx(ctx)
+            code = yield from execute([fpath, ctx['src_name'],
+                                       ctx['version'], ctx['debian_revision']],
+                                      cwd=dpath)
+            if code != 0:
+                raise Exception("pre hook for %s failed with %d return code" % (
+                                ctx['name'], code))
+    for o_dpath in reversed(override_paths):
+        # copy static files
+        deb_dpath = join(o_dpath, 'debian')
+        if isdir(deb_dpath):
+            _copy_static_files(deb_dpath, debian_dir)
 
     for key in ('vcs_src', 'vcs_browser', 'uploaders'):
         if key in ctx:
             ctx[key] = ctx[key].format(**ctx)
+    ctx['debian_version'] = "{}-{}".format(ctx['version'], ctx['debian_revision'])
 
-    # copy static files
-    debian_dir = join(dpath, 'debian')
-    override_path = join(OVERRIDES_PATH, ctx['name'].lower())
-    override_dpath = join(override_path, 'debian')
-    _copy_static_files(override_dpath, debian_dir)
-    _copy_static_files(join(TEMPLATES_PATH, 'debian'), debian_dir)
-    if profile:
-        _copy_static_files(join(PROFILES_PATH, profile, 'debian'), debian_dir)
-
-    env = Environment(loader=FileSystemLoader([dpath, override_path,
-                                               TEMPLATES_PATH]))
+    # Jinja setup: set templates directories
+    templates_dir = [dpath]  # use existing dir as a template dir as well
+    templates_dir.extend(override_paths)
+    templates_dir.append(TEMPLATES_PATH)
+    env = Environment(loader=FileSystemLoader(templates_dir))
 
     # render debian dir files (note that order matters)
     docs(dpath, ctx, env)
     control(dpath, ctx, env)
     rules(dpath, ctx, env)
+    chmod(join(dpath, 'debian', 'rules'), 0o755)
     initial_release = yield from changelog(dpath, ctx, env)
     if initial_release:
         itp_mail(dpath, ctx, env)
@@ -110,6 +137,18 @@ def debianize(dpath, ctx, profile=None):
     watch(dpath, ctx, env)
     clean(dpath, ctx, env)
 
+    # invoke post hooks
+    for o_dpath in override_paths:
+        fpath = join(o_dpath, 'hooks', 'post')
+        if exists(fpath) and access(fpath, X_OK):
+            _dump_ctx(ctx)
+            code = yield from execute([fpath, ctx['src_name'],
+                                       ctx['version'], ctx['debian_revision']],
+                                      cwd=dpath)
+            if code != 0:
+                raise Exception("post hook for %s failed with %d return code" % (
+                                ctx['name'], code))
+
 
 def update_ctx(dpath, ctx):
     ctx.setdefault('exports', {})
@@ -139,6 +178,18 @@ def update_ctx(dpath, ctx):
                             ctx['clean_files'].add(join(root.replace(dpath, '.'), fname_c))
 
 
+def _dump_ctx(ctx):
+    """dump ctx in JSON format so that hooks can use it"""
+    try:
+        fpath = join(ctx['root'], '{src_name}_{version}-{debian_revision}.ctx'.format(**ctx))
+        serializable_ctx = {key: (value if not isinstance(value, set) else tuple(value))
+                            for (key, value) in ctx.items()}
+        with open(fpath, 'w') as fp:
+            dump(serializable_ctx, fp, indent=' ')
+    except Exception:
+        log.debug('canont dump ctx', exc_info=True)
+
+
 def docs(dpath, ctx, env):
     docs = ctx.setdefault('docs', {})
     for path in SPHINX_DIR:
@@ -190,7 +241,14 @@ def _render_template(func):
 def control(dpath, ctx, env):
     desc = []
     code_line = False
+    first_line = True
     for line in ctx['description'].split('\n'):
+        if first_line:
+            if line.lower() == ctx['name'].lower():
+                continue
+            if not line.strip().replace('=', '').replace('-', '').replace('~', ''):
+                continue
+            first_line = False
         if not line.strip():
             desc.append(' .')
         else:
@@ -202,6 +260,8 @@ def control(dpath, ctx, env):
                 elif code_line:
                     code_line = False
                 line = ' ' + line
+            elif line.strip().lower() in DESC_STOP_KEYWORDS:
+                break
             line = line.replace('\t', '    ')
             desc.append(' ' + line)
     for key, value in {
@@ -281,7 +341,7 @@ def rules(dpath, ctx, env):
 
 def changelog(dpath, ctx, env):
     change = ctx.get('message', 'Autogenerated by py2dsp v{}'.format(VERSION))
-    version = "{}-{}".format(ctx['version'], ctx['debian_revision'])
+    version = ctx['debian_version']
     distribution = ctx.get('distribution', 'UNRELEASED')
 
     fpath = join(dpath, 'debian', 'changelog')
@@ -315,11 +375,25 @@ def changelog(dpath, ctx, env):
 
 @_render_template
 def copyright(dpath, ctx, env):
-    ctx['deb_copyright'] = "2015 © {}".format(ctx['creator'])
-    ctx['deb_license_name'] = ctx['license_name']
+    if not ctx.get('deb_copyright'):
+        ctx['deb_copyright'] = "{} © {}".format(datetime.now().year, ctx['creator'])
+    if ctx['license_name'] == 'Apache 2':
+        ctx['license_name'] = 'Apache2'
+        ctx['license'] = ' See /usr/share/common-licenses/Apache-2.0'
+    if not ctx.get('deb_license_name'):
+        ctx['deb_license_name'] = ctx['license_name']
+        ctx['deb_license'] = ctx.get('license', '')
+    if not ctx.get('deb_license') and ctx['deb_license_name']:
+        ctx['deb_license'] = ''
+        fpath = '/usr/share/common-licenses/{}'.format(ctx['deb_license_name'])
+        if exists(fpath):
+            license = []
+            with open(fpath, encoding='utf-8') as fp:
+                for line in fp:
+                    line = line.rstrip()
+                    license.append(' {}'.format(line) if line else ' .')
+            ctx['deb_license'] = '\n'.join(license)
 
-    if exists('/usr/share/common-licenses/{}'.format(ctx['license_name'])):
-        pass  # FIXME
     ctx.setdefault('copyright', '')
 
     if not ctx.get('license'):
@@ -329,8 +403,8 @@ def copyright(dpath, ctx, env):
                 continue
             with open(join(dpath, fn), 'r') as fp:
                 for line in fp:
-                    line = line.strip('\n')
-                    if not line.strip():
+                    line = line.rstrip()
+                    if not line:
                         license.append(' .')
                     else:
                         license.append(' ' + line)
